/*
 * xtags - print C-file's indexes
 *
 * Program zaliczeniowy
 * Autor: Micha Moskal (nr albumu 129934)
 *
 * $Id: main.c,v 1.4 2000/10/24 23:25:36 malekith Exp $
 */

#include "xtags.h"

/* flags : */
int print_flags = t_func | t_global | t_cpp;
int verbose;
int allow_plus_comments;
int allow_plus_keywords;

/* params from command line */
static const char **files;
static int files_count;

static const char *out_file;

const char *our_name;	/* argv[0] */

static void parse_options(int argc, const char **argv);

static const char *help =
"USAGE: %s [-flags] [-o outfile] [file...]\n"
"  What to print:\n"	
"	-f - don't include functions\n"
"	-g - don't include global objects (variables, structs)\n"
"	-d - don't include #defines\n"
"	-l - include local variables\n"
"	-k - include keywords\n"
"	-a - print everthing...\n"
"  Syntax:\n"
"	-p - allow C++ style one-line comments (//)\n"
"	-P - allow C++ keywords\n"
"  General:\n"
"	-v - be verbose, use twice to double effect\n"
"	-h - display this help\n"
"	-? -       - \" -\n"
"\n"
"  Output format example:\n"
"	- func() [123]: 134, 1113.\n"
"	- cpp_sym# [112]: 1112, 1112.\n"
"	- var [12]: 34.\n"
"	- printf [0]: 12, 112, 115.\n"
"   This  donates:  function  named  `func' defined in 123rd line and\n"
"   used in 134th and 1113th  lines,  preprocessor  symbol  `cpp_sym'\n"
"   #defined  in  112th  line and used two times in 1112th line, some\n"
"   variable or structure named `var', defined in 12th line and  used\n"
"   in  34th,  and  finnaly  function `printf' not defined in current\n"
"   file (hence [0]) and used in 12th, 112th and 115th lines of  cur-\n"
"   rent file...  uff... ;)\n"
"\n"
"   Note  that places of defintion are only best gueses, and creative\n"
"   uses of #defines can drive this simple program mad...\n"
;


static const char *req_param = "%s: option -%c requires parameter\n";

static void parse_options(int argc, const char **argv)
{
	const char *p;
	const char *arg = 0;
	int c;
	
	argc--;
	our_name = *argv++;

	while (argc--) {
		p = *argv++;

		/* "-" means stdin ... */
		if (*p != '-' || strcmp(p, "-") == 0) {
			files = --argv;
			files_count = ++argc;
			return;
		}

		for (p++; *p; p++) {
			c = *p;
			if (strchr("o", *p)) {
				if (p[1]) { 
					arg = p + 1;
					p = strchr(p, 0);
				} else if (argc) {
					arg = *argv++;
					argc--;
				} else {
					fprintf(stderr, req_param, 
						our_name, *p);
					exit(1);
				}
			}
			
			switch (c) {
			case 'P':
				allow_plus_keywords++;
				break;
			case 'p':
				allow_plus_comments++;
				break;
			case 'o':
				out_file = arg;
				break;
			case 'a':
				print_flags = 0xffff;
				break;
			case 'f':
				print_flags &= ~t_func;
				break;
			case 'g':
				print_flags &= ~t_global;
				break;
			case 'd':
				print_flags &= ~t_cpp;
				break;
			case 'k':
				print_flags |= t_keyword;
				break;
			case 'l':
				print_flags |= t_local;
				break;
			case 'v':
				verbose++;
				break;
			case 'h':
			case '?':
				printf(help, our_name);
				exit(0);
			default:
				printf(help, our_name);
				exit(1);
			}
		}
	}
}

int main(int argc, const char **argv)
{
	FILE *in, *out;
	
	parse_options(argc, argv);

	if (verbose) {
		fprintf(stderr, "flags 0x%04x\noutput = %s\n",
			print_flags,
			out_file);
	}

	if (out_file) {
		out = fopen(out_file, "w");
		if (out == 0) {
			perror(out_file);
			exit(1);
		}
	} else 
		out = stdout;

	if (files_count == 0)
		parse(stdin, out, "stdin");
	else
		while (files_count--) {
			in = fopen(*files, "r");
			if (in == 0) {
				perror(*files);
				exit(1);
			}
			parse(in, out, *files++);
			fclose(in);
		}

	if (out_file)
		fclose(out);
		
	return 0;
}
